<?php

namespace SGalinski\SgJobs\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgJobs\Domain\Model\Company;
use TYPO3\CMS\Core\Context\Context;
use TYPO3\CMS\Core\Context\Exception\AspectNotFoundException;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Persistence\QueryInterface;
use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;
use TYPO3\CMS\Extbase\Persistence\Repository;

/**
 * Company Repository
 */
class CompanyRepository extends Repository {
	/**
	 * initializes the object
	 */
	public function initializeObject(): void {
		$querySettings = $this->createQuery()->getQuerySettings();
		$querySettings->setRespectStoragePage(FALSE);
		$querySettings->setIgnoreEnableFields(TRUE);
		$querySettings->setEnableFieldsToBeIgnored(['disabled']);
		$this->setDefaultQuerySettings($querySettings);
	}

	/**
	 * Returns all countries filtered by page id
	 *
	 * @param int $pageUid
	 * @param array $filters
	 * @return array
	 * @throws AspectNotFoundException
	 */
	public function getAllCountries(int $pageUid, array $filters = []) {
		$result = $this->getAllCompanies($pageUid, $filters)->toArray();

		$countryArray = [];
		/** @var Company $company */
		foreach ($result as $company) {
			$countryName = $company->getCountry();
			if ($countryName !== '') {
				// The filtering needs this specific key for some reason.
				$countryArray[$countryName] = $countryName;
			}
		}

		return $countryArray;
	}

	/**
	 * Returns all filtered cities
	 *
	 * @param int $pageUid
	 * @param array $filters
	 * @return array
	 * @throws AspectNotFoundException
	 */
	public function getAllCities(int $pageUid, array $filters = []) {
		$result = $this->getAllCompanies($pageUid, $filters)->toArray();

		$cityArray = [];
		/** @var Company $company */
		foreach ($result as $company) {
			$cityName = $company->getCity();
			if ($cityName !== '') {
				// The filtering needs this specific key for some reason.
				$cityArray[$cityName] = $cityName;
			}
		}

		return $cityArray;
	}

	/**
	 * Returns all company names filtered by page id
	 *
	 * @param int $pageUid
	 * @param array $filters
	 * @return array
	 * @throws AspectNotFoundException
	 */
	public function getAllCompanyNames(int $pageUid, array $filters = []) {
		$result = $this->getAllCompanies($pageUid, $filters)->toArray();

		$companyArray = [];
		/** @var Company $company */
		foreach ($result as $company) {
			$companyName = $company->getName();
			if ($companyName !== '') {
				// The filtering needs this specific key for some reason.
				$companyArray[$companyName] = $companyName;
			}
		}

		return $companyArray;
	}

	/**
	 * Returns all companies filtered by page id
	 *
	 * @param int $pageUid
	 * @param array $filters
	 * @return object[]|QueryResultInterface
	 * @throws AspectNotFoundException
	 */
	public function getAllCompanies(int $pageUid, array $filters = []) {
		$companyConstraints = [];
		$query = $this->createQuery();
		$querySettings = $query->getQuerySettings();
		$querySettings->setStoragePageIds([$pageUid]);
		$querySettings->setLanguageUid(
			GeneralUtility::makeInstance(Context::class)->getPropertyFromAspect('language', 'id', 0)
		);

		$query->setQuerySettings($querySettings);

		$query->setOrderings(
			[
				'sorting' => QueryInterface::ORDER_ASCENDING,
			]
		);

		$companyConstraints[] = $query->equals('pid', $pageUid);

		if (count($filters) > 0) {
			$constraints = [];
			foreach ($filters as $filter) {
				$constraints[] = $query->equals('uid', $filter);
			}
			$companyConstraints[] = $query->logicalOr($constraints);
		}

		$query->matching($query->logicalAnd($companyConstraints));

		return $query->execute();
	}
}
